import argparse


def convert_snapshot_to_density(snapshot_path, output_path, N=None, corner=(0.0, 0.0, 0.0), time=1.0):
    """
    Convert a snapshot to a density field.

    Parameters
    ----------
    snapshot_path : str
        Path to the snapshot file.
    output_path : str
        Path to the output density file.
    N : int
        Size of the density field grid (N x N x N).
    corner : tuple of float
        Corner of the box (x, y, z).
    """
    from pysbmy.density import get_density_pm_snapshot
    from pysbmy.snapshot import read_snapshot

    # Read the snapshot
    print("Reading snapshot...")
    snap = read_snapshot(snapshot_path)

    if N is None:
        N = snap.Np0

    # Calculate density
    print("Calculating density...")
    F = get_density_pm_snapshot(snap, N, N, N, corner[0], corner[1], corner[2])
    F.time = time  # Set the time for the field

    # Write density to file
    print("Writing density...")
    F.write(output_path)
    print("Density written to", output_path)
    print("Done.")
    

def convert_snapshots_to_density(snapshot_base_path, output_path, N=None, corner=(0.0, 0.0, 0.0), time=1.0):
    """
    Convert multiple snapshots to density fields.

    Parameters
    ----------
    snapshot_base_path : str
        Base path for the snapshot files.
    output_path : str
        Path to the output density file.
    N : int
        Size of the density field grid (N x N x N).
    corner : tuple of float
        Corner of the box (x, y, z).
    """
    from pysbmy.density import get_density_pm, density_to_delta
    from pysbmy.snapshot import read_snapshot
    from pysbmy.field import Field
    import numpy as np
    
    # Get all snapshot files with path "snapshot_base_path*"
    import glob
    
    snapshot_files = glob.glob(snapshot_base_path + "*")
    if not snapshot_files:
        raise FileNotFoundError(f"No snapshot files found at {snapshot_base_path}")
    
    A = None
    L0 = L1 = L2 = None
    d0 = d1 = d2 = None
    
    for snapshot_path in snapshot_files:
        # Read the snapshot
        print(f"Reading snapshot {snapshot_path}...")
        snap = read_snapshot(snapshot_path)
        
        if N is None:
            N = snap.Np0
        
        if L0 is None:
            # Get the grid parameters from the snapshot
            L0 = snap.L0
            L1 = snap.L1
            L2 = snap.L2
            d0 = L0 / N
            d1 = L1 / N
            d2 = L2 / N
        else:
            # Ensure the grid parameters match the first snapshot
            if not (snap.L0 == L0 and snap.L1 == L1 and snap.L2 == L2):
                raise ValueError(f"All snapshots must have the same grid parameters. Got {snap.L0}, {snap.L1}, {snap.L2} but expected {L0}, {L1}, {L2}.")
        
        if A is None:
            # Initialize the density field
            A = np.zeros((N, N, N), dtype=np.float32)

        # Calculate density
        print("Calculating density for this snapshot...")
        A = get_density_pm(snap.pos, A, d0=d0, d1=d1, d2=d2)
    
    # Convert density to delta field
    print("Converting density to delta field...")
    A = density_to_delta(A)
    
    # Create a Field object
    F = Field(L0=L0,L1=L1,L2=L2,corner0=corner[0],corner1=corner[1],corner2=corner[2],rank=1,N0=N,N1=N,N2=N,time=time,data=A)
    # Write density to file
    print("Writing density...")
    F.write(output_path)
    print("Density written to", output_path)
    print("Done.")


def console_main():
    parser = argparse.ArgumentParser(description="Convert snapshot to density.")
    parser.add_argument(
        "-S",
        "--snapshot",
        type=str,
        required=True,
        help="Path to the snapshot file.",
    )
    parser.add_argument(
        "-o",
        "--output",
        type=str,
        required=True,
        help="Path to the output density file.",
    )
    parser.add_argument(
        "-N",
        "--N",
        type=int,
        default=None,
        help="Size of the density field grid (N x N x N).",
    )
    parser.add_argument(
        "-c",
        "--corner",
        type=float,
        nargs=3,
        default=[0.0, 0.0, 0.0],
        help="Corner of the box (x, y, z).",
    )

    args = parser.parse_args()

    if args.snapshot.endswith("h5") or args.snapshot.endswith("hdf5") or args.snapshot.endswith("gadget3"):
        # If the snapshot is a single file, convert it to density
        convert_snapshot_to_density(
            snapshot_path=args.snapshot,
            output_path=args.output,
            N=args.N,
            corner=args.corner,
        )
    else:
        # If the snapshot is a base path, convert all snapshots to density
        convert_snapshots_to_density(
            snapshot_base_path=args.snapshot,
            output_path=args.output,
            N=args.N,
            corner=args.corner,
        )


if __name__ == "__main__":
    console_main()
